﻿using Microsoft.Extensions.Logging;
using ProjectMotorboat.CollectionGenericObjects;
using ProjectMotorboat.Drownings;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace ProjectMotorboat;

public partial class FormBoatCollection : Form
{
    private readonly StorageCollection<DrawningBoat> _storageCollection;

    private AbstractCompany? _company = null;

    private readonly ILogger _logger;

    public FormBoatCollection(ILogger<FormBoatCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
        _logger.LogInformation("Форма загрузилась");
    }


    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }


    private void ButtonAddBoat_Click(object sender, EventArgs e)
    {
        FormBoatConfig form = new();
        form.Show();
        form.AddEvent(SetBoat);

    }
    private void SetBoat(DrawningBoat boat)
    {
        if (_company == null || boat == null)
        {
            return;
        }
        try
        {
            var res = _company + boat;
            MessageBox.Show("Объект добавлен");
            _logger.LogInformation($"Объект добавлен под индексом {res}");
            pictureBox.Image = _company.Show();
        }
        catch (Exception ex)
        {
            MessageBox.Show($"Объект не добавлен: {ex.Message}", "Результат", MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
        }
    }


    private void ButtonRemoveBoat_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
            DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            var res = _company - pos;
            MessageBox.Show("Объект удален");
            _logger.LogInformation($"Объект удален под индексом {pos}");
            pictureBox.Image = _company.Show();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Не удалось удалить объект",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
        }
    }



    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningBoat? boat = null;
        int counter = 100;
        while (boat == null)
        {
            boat = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }
        if (boat == null)
        {
            return;
        }
        FormMotorboat form = new()
        {
            SetBoat = boat
        };
        form.ShowDialog();
    }


    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) ||
            (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }

        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }

        try
        {
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            _logger.LogInformation("Добавление коллекции");
            RerfreshListBoxItems();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
        }

    }

    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
            DialogResult.Yes)
        {
            return;
        }

        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        _logger.LogInformation("Коллекция удалена");
        RerfreshListBoxItems();
    }

    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }

    }
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        ICollectionGenericObjects<DrawningBoat>? collection =
            _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new HarborService(pictureBox.Width, pictureBox.Height, collection);
                _logger.LogInformation("Компания создана");
                break;
        }

        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                RerfreshListBoxItems();
                MessageBox.Show("Загрузка прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Загрузка не выполнена", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

}
